org 100h
bits 16
cpu 8086

num_cols equ 16

section .text

start:
	mov bl, [080h] ; command-line arguments length
	and bl, bl
	jz exit
	xor bh, bh
	mov byte [081h + bx], 0
	; Now 082h is usable for filename (zero-terminated string)

	mov ah, 09h
	mov dx, ckfile_str
	int 21h
	mov dx, 082h
	call putstring
	call newline

	; Open file
	mov ah, 03dh
	mov al, 00 ; read only
	mov dx, 082h
	int 21h
	jc error
	mov [fhandle], ax


	mov word [bsd_chksum], 0 ; Initialize checksum
	mov word [sysv_chksum], 0 ; Initialize checksum

	; Reading
readchunk:
	mov ah, 03fh
	mov bx, [fhandle]
	mov cx, num_cols
	mov dx, readbuf
	int 21h ; n bytes read in ax

	; Display row
	mov bx, readbuf
	mov cx,ax
	and cx,cx ; check for zero-byte read
	jz no_more_data
next:
	;; BDC checksum part
	mov ax, [bx]		; Get byte
	xor ah, ah			; Keep low value
	mov dx, ax			; Store in dx
	mov ax, [bsd_chksum]	; Get current checksum
	ror ax, 1
	add ax, dx			; Add the byte
	mov [bsd_chksum], ax	; Store

	;; SYSV checksum part
	mov ax, [bx]		; Get byte
	xor ah, ah			; Keep low value
	mov dx, ax			; Store in dx
	mov ax, [sysv_chksum]	; Get current checksum
	;ror ax, 1
	add ax, dx			; Add the byte
	mov [sysv_chksum], ax	; Store

	inc bx
	dec cl
	jnz next

	jmp readchunk

no_more_data:
	; Close the file
	mov ah, 03eh
	mov bx, [fhandle]
	int 21h

	; Print BSD checksum
	mov ah, 09h
	mov dx, bsd_str
	int 21h

	mov ax, [bsd_chksum]
	mov dl, ah
	call printHexByte
	mov dl, al
	call printHexByte
	call newline

	; Print sysv checksum
	mov ah, 09h
	mov dx, sysv_str
	int 21h

	mov ax, [sysv_chksum]
	mov dl, ah
	call printHexByte
	mov dl, al
	call printHexByte
	call newline


	jmp exit

error:
	mov ah, 09h
	mov dx, error_str
	int 21h

exit:
	mov ah,04CH
	mov al,00 ; Return 0
	int 21h

newline:
	push ax
	push dx
	mov ah,09h
	mov dx, nl_str
	int 21h
	pop dx
	pop ax
	ret

printSpace:
	push ax
	push dx
	mov ah,02h
	mov dl, ' '
	int 21h
	pop dx
	pop ax
	ret

; ***** printHexByte
; Byte in dl
printHexByte:
	push dx
	;shr dl, 4 ; Not available on 8086
	shr dl, 1
	shr dl, 1
	shr dl, 1
	shr dl, 1
	call printHexNibble
	pop dx
	call printHexNibble
	ret


; ***** printhexnibble
; Argument : Value in DL
printHexNibble:
	push dx
	and dl, 0FH
	cmp dl, 9
	ja letters
	add dl, '0'
	call putchar
	pop dx
	ret
letters:
	add dl, 'A'-10
	call putchar
	pop dx
	ret

; ******* putstring ******
; Argument : NUL-terminated string in DX
putstring:
	mov bx,dx
	push ax
	mov ah,02H
putstring_loop:
	mov dl, [bx]
	cmp dl,0
	jz putstring_done
	int 21h
	inc bx
	jmp putstring_loop
putstring_done:
	pop ax
	ret

; ******* putchar ******
; Argument: Character in DL
; Return: None
putchar:
	push ax
	mov ah,02H
	int 21h
	pop ax
	ret


section .data

ckfile_str:		db 'Checksumming file: $'
nl_str:			db 13,10,'$'
error_str:		db 13,10,'Error!',13,10,'$'
bsd_str:		db 'BSD checksum: $'
sysv_str:		db 'SYSV checksum: $'

section .bss

bsd_chksum: resw 1
sysv_chksum: resw 1
fhandle: resw 1
readbuf: resb num_cols
