org 100h
bits 16
cpu 8086

;%define DEBUG

section .text

start:
	mov ah,09H
	mov dx,msg_hello
	int 21h

	jmp main

main:
initport:
	mov al, 0
	mov dx, 0378H
	out dx, al
	mov al, 1
	out dx, al

	;;; Ready
	mov ah,09H				;
	mov dx,msg_waitblock	;
	int 21h					;

waitHeader:
	call waitBlockAvailable
	jc exit ; Keyboard interrupt
	mov bx, header
	call receiveBlock

	; Check for magic number and version (R1)
	mov ax, [header]
	cmp ax, 03152h
	je valid_header
	mov ah, 09h
	mov dx, msg_ignored_invalid_header
	int 21h
	jmp waitHeader

valid_header:
	; Show Filename: xxxxxxxx.xxx
	mov ah, 09h
	mov dx, msg_filename
	int 21h
	mov dx, header+2
	call putstring
	call newline
	; Show xx blocks and n bytes
	mov dl, [header+16]
	call printHexByte
	mov dl, [header+15]
	call printHexByte
	mov dx, msg_blocks
	int 21h
	mov dl, [header+17]
	call printHexByte
	mov dx, msg_bytes
	int 21h

	; Now prepare the file

	; Create file
	mov ah, 03ch
	mov dx, header+2
	mov cx, 0h
	int 21h
	jc error
	mov [fhandle], ax

	; Open file
;	mov ah, 03dh
;	mov al, 01 ; write only
;	mov dx, header+2
;	int 21h
;	jc error
;	mov [fhandle], ax

	mov word [written_blocks], 0
mainloop:
	; Check where we are
	mov ax, [written_blocks]
	cmp ax, [header+15]
	je maybe_partial

	inc ax
	mov [written_blocks], ax
	jmp full_block

maybe_partial:
	; If the file was a multiple of 32, there
	; won't be other blocks for us.
	mov cx, [header+17]
	xor ch,ch
	and cl,cl
	jz nopartial

	; Wait for a block to be available
	call waitBlockAvailable
	jc exit ; Keyboard interrupt

	; Receive the block
	mov bx, rx_buffer
	call receiveBlock
	call progressIndicator

	; partial (and last) block
	call writeBlock
nopartial:
	call closeFile
	mov ah, 09h
	mov dx, msg_done
	int 21h
	jmp main ; Go wait for next file

full_block:
	; Wait for a block to be available
	call waitBlockAvailable
	jc exit ; Keyboard interrupt

	; Receive the block
	mov bx, rx_buffer
	call receiveBlock
	call progressIndicator

	mov cx, 32
	call writeBlock
	jmp mainloop

closeFile:
	push ax
	push bx
	mov ah, 03eh
	mov bx, [fhandle]
	int 21h
	jc error
	pop bx
	pop ax
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Write the block in rx_buffer to file.
; Number of bytes to write passed in cx
writeBlock:
	push ax
	push bx
	push dx

%ifdef DEBUG ; Be verbose
	mov ah,09H
	mov dx,msg_writing
	int 21h
%endif

	mov ah, 0040h
	mov bx, [fhandle]
;	mov cx, 32
	mov dx, rx_buffer
	int 21h
	pop dx
	pop bx
	pop ax
	ret

; Wait until a block is available. Can be interrupted
; by ESC. When interrupted, CF set at return.
waitBlockAvailable:
	push ax
	push dx

waitBlockAvailable_loop:
	; check for keys in STDIN
	mov ah, 00Bh
	int 21h
	and al,al
	jz no_keys

	; If ESC was pressed, exit
	mov ah, 008h
	int 21h
	cmp al, 27
	clc
	cmc
	pop dx
	pop ax
	ret

no_keys:
	mov dx, 0378H
	mov al, 1
	out dx, al
	mov al, 0
	out dx, al

	call delay
	call getNibble
	cmp al, 0
	jz waitBlockAvailable_loop
	pop dx
	pop ax
	clc
	ret

; Receive a full 32-byte block
; Receive buffer pointer in BX
receiveBlock:
	push ax
	push bx
	push cx
	push dx

%ifdef DEBUG
	mov ah,09h
	mov dx,msg_receiving_block
	int 21h
%endif

	mov cl, 32
	;mov bx, rx_buffer
next_byte:
	call getByte

%ifdef DEBUG
	mov dl, al
	call printHexByte
%endif

	mov [bx], al
	inc bx
	dec cl
	jnz next_byte

	pop dx
	pop cx
	pop bx
	pop ax

	ret


; Return a byte using the 2/3 handshake to get the high and low nibble.
; Byte returned in al
;
getByte:
	push dx
	push cx
	mov dx, 0378H

	mov al, 2
	out dx, al
	; delay?
	call delay

	call getNibble
	mov cl, al
	shl cl, 1
	shl cl, 1
	shl cl, 1
	shl cl, 1

	mov al, 3
	out dx, al
	; delay?
	call delay

	call getNibble
	or al, cl

	pop cx
	pop dx
	ret

;	Port 379h bits 7 6 5 and 3 packed right
;	Returned in al
getNibble:
	push dx
	mov dx, 0379H
	in al, dx
	mov ah, al
	and ah, 0E0H
	and al,  08H
	shr ah, 1
	shr ah, 1
	shr ah, 1
	shr ah, 1
	shr al, 1
	shr al, 1
	shr al, 1
	or al, ah
	pop dx
	ret

delay:
	push dx
	push ax
	mov dx, 0378H
	; Note: This delay does not really seem necessary, and removing
	; the block below does not make a measurable difference in transfer
	; rate. But I feel it's safer to leave it there.
	in al, dx
	in al, dx
	in al, dx
	in al, dx
	in al, dx
	in al, dx
	in al, dx
	in al, dx
	in al, dx
	in al, dx
	in al, dx
	in al, dx
	in al, dx
	in al, dx
	in al, dx
	in al, dx
	in al, dx
	in al, dx
	in al, dx
	in al, dx
	in al, dx
	in al, dx
	in al, dx
	in al, dx
	pop ax
	pop dx
	ret


	jmp exit

error:
	mov dl, al ; int21 error core in AXS
	push dx
	call newline
	call newline
	mov ah,09H
	mov dx,errormsg
	int 21h
	pop dx
	call printHexByte
	call newline
	jmp exit

; ***** printHexByte
; Byte in dl
printHexByte:
	push dx
	;shr dl, 4 ; Not available on 8086
	shr dl, 1
	shr dl, 1
	shr dl, 1
	shr dl, 1
	call printHexNibble
	pop dx
	call printHexNibble
	ret

; ***** printhexnibble
; Argument : Value in DL
printHexNibble:
	push dx
	and dl, 0FH
	cmp dl, 9
	ja letters
	add dl, '0'
	call putchar
	pop dx
	ret
letters:
	add dl, 'A'-10
	call putchar
	pop dx
	ret


; ******* putstring ******
; Argument : NUL-terminated string in DX
putstring:
	mov bx,dx
	push ax
	mov ah,02H
putstring_loop:
	mov dl, [bx]
	cmp dl,0
	jz putstring_done
	int 21h
	inc bx
	jmp putstring_loop
putstring_done:
	pop ax
	ret

; ******* putchar ******
; Argument: Character in DL
; Return: None
putchar:
	push ax
	mov ah,02H
	int 21h
	pop ax
	ret

progressIndicator:
%ifdef DEBUG
	call newline
	ret
%endif
	push ax
	push dx
	mov ah,02H
	mov dl,'.'
	int 21h
	pop dx
	pop ax
	ret

newline:
	push ax
	push dx
	mov ah, 09H
	mov dx, newline_str
	int 21h
	pop dx
	pop ax
	ret

exit:
	mov ah, 09h
	mov dx, msg_exiting
	int 21h

	mov ah,04CH
	mov al,00 ; Return 0
	int 21h


section .data

msg_hello: 		db 'RXFILES v0.1 by Raphael Assenat <raph@raphnet.net>',13,10,'$'
msg_waitblock: 	db 'Ready (press ESC to stop)',13,10,'$'
msg_exiting: 	db 13,10,'Exiting',13,10,'$'
msg_filename: 	db 'Filename: $'
msg_blocks: 	db 'h blocks and $'
msg_bytes: 		db 'h byte(s).$'
msg_done: 		db 'Done',13,10,'$'
msg_ignored_invalid_header: db 'INFO: Ignored invalid header',13,10,'$'
%ifdef DEBUG
msg_receiving_block: db 'Receiving block...',13,10,'$'
msg_writing: 	db 'writing to file...$'
%endif
errormsg:		db 'Error $'
newline_str:	db 13,10,'$'

section .bss

written_blocks:	resw 1
fhandle:		resw 1
rx_buffer:		resb 32
header:			resb 32


