#ifndef _raphnethid_h__
#define _raphnethid_h__

#include <wchar.h>

#define OUR_VENDOR_ID 	0x289b
#define PRODNAME_MAXCHARS	256
#define SERIAL_MAXCHARS		256
#define PATH_MAXCHARS		256

#define PRODUCT_GCN64_USB_V3	0
#define PRODUCT_PPUSBCOMM		1

struct raphnethid_info {
	int product;
	wchar_t str_prodname[PRODNAME_MAXCHARS];
	wchar_t str_serial[SERIAL_MAXCHARS];
	char str_path[PATH_MAXCHARS];
	int usb_vid, usb_pid;
	int major, minor;
	int access; // True unless direct access to read serial/prodname failed due to permissions.
};

struct raphnethid_list_ctx;

typedef void* raphnethid_hdl_t; // Cast from hid_device

int raphnethid_init(int verbose);
void raphnethid_shutdown(void);

struct raphnethid_list_ctx *raphnethid_allocListCtx(void);
void raphnethid_freeListCtx(struct raphnethid_list_ctx *ctx);
struct raphnethid_info *raphnethid_listDevices(struct raphnethid_info *info, struct raphnethid_list_ctx *ctx);

raphnethid_hdl_t raphnethid_openDevice(struct raphnethid_info *dev);

#define GCN64_FLG_OPEN_BY_SERIAL	1	/** Serial must match */
#define GCN64_FLG_OPEN_BY_PATH		2	/** Path must match */
#define GCN64_FLG_OPEN_BY_VID		4	/** USB VID must match */
#define GCN64_FLG_OPEN_BY_PID		8	/** USB PID MUST match */
/**
 * \brief Open a device matching a serial number
 * \param dev The device structure
 * \param flags Flags 
 * \return A handle to the opened device, or NULL if not found
 **/
raphnethid_hdl_t raphnethid_openBy(struct raphnethid_info *dev, unsigned char flags);

void raphnethid_closeDevice(raphnethid_hdl_t hdl);

int raphnethid_send_cmd(raphnethid_hdl_t hdl, const unsigned char *cmd, int len);
int raphnethid_poll_result(raphnethid_hdl_t hdl, unsigned char *cmd, int cmdlen);

int raphnethid_exchange(raphnethid_hdl_t hdl, unsigned char *outcmd, int outlen, unsigned char *result, int result_max);

#endif // _raphnethid_h__

